<?php

namespace Drupal\image_sizes\Plugin\Field\FieldFormatter;

use Drupal\Component\Utility\Html;
use Drupal\Core\Field\FieldItemInterface;
use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Field\FormatterBase;
use Drupal\Core\Form\FormStateInterface;

use Drupal\file\Entity\File;
use Drupal\image\Entity\ImageStyle;


/**
 * Plugin implementation of the 'image_sizes_formatter' formatter.
 *
 * @FieldFormatter(
 *   id = "image_sizes_formatter",
 *   label = @Translation("Image sizes formatter"),
 *   field_types = {
 *     "image"
 *   }
 * )
 */
class ImageSizesFormatter extends FormatterBase {

  /**
   * {@inheritdoc}
   */
  public static function defaultSettings() {
    return [
      // Implement default settings.
      'image_style_placeholder' => false,
      'image_styles' => [],
      'fallback_style' => 'original'
    ] + parent::defaultSettings();
  }

  /**
   * {@inheritdoc}
   */
  public function settingsForm(array $form, FormStateInterface $form_state) {
    return [
      // Implement settings form.
      'image_style_placeholder' => [
        '#type' => 'select',
        '#required' => true,
        '#title' => $this->t('Placeholder image style.'),
        '#options' => image_style_options(),
        '#default_value' => $this->getSetting('image_style_placeholder')
      ],
      'image_styles' => [
        '#type' => 'select',
        '#multiple' => true,
        '#required' => true,
        '#options' => $this->getOptions(),
        '#title' => $this->t('Choose image styles for each width.'),
        '#default_value' => $this->getSetting('image_styles')
      ],
      'fallback_style' => [
        '#type' => 'select',
        '#required' => true,
        '#title' => $this->t('Fallback image if no style is selectable.'),
        '#options' => array_merge(
          ['original' => t('Original')],
          image_style_options(false)
        ),
        '#default_value' => $this->getSetting('fallback_style')
      ]
    ] + parent::settingsForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function settingsSummary() {
    $summary = [];
    // Implement settings summary.

    return $summary;
  }

  /**
   * {@inheritdoc}
   */
  public function viewElements(FieldItemListInterface $items, $langcode) {
    $elements = [];
    foreach ($items as $delta => $item) {
      $elements[$delta] = $this->viewValue($item);
    }
    $elements['#attached']['library'] = ['image_sizes/core'];

    return $elements;
  }

  protected function getOptions() {
    $image_styles = \Drupal::entityQuery('image_style')
      ->execute();
    $styles = [];
    foreach( $image_styles as $style_name ) {
      $style = ImageStyle::load($style_name);
      $effects = $style->getEffects();
      foreach ($effects as $effect) {
        if (is_a($effect, 'Drupal\image\Plugin\ImageEffect\ResizeImageEffect')) {
          $width = $effect->getConfiguration()['data']['width'];
          if ($width !== null && $width > 0) {
            $styles[$style_name] = $style->label() . " ($width)";
          }
        }
      }
    }
    return $styles;
  }

  protected function getStyles($uri) {
    $image_styles = \Drupal::entityQuery('image_style')
      ->condition('name' , $this->getSetting('image_styles'), 'IN')
      ->execute();
    $styles = [];
    foreach( $image_styles as $style_name ) {
      $style = ImageStyle::load($style_name);
      $effects = $style->getEffects();
      foreach ($effects as $effect) {
        if (is_a($effect, 'Drupal\image\Plugin\ImageEffect\ResizeImageEffect')) {
          $width = $effect->getConfiguration()['data']['width'];
          if ($width !== null && $width > 0) {
            $styles[$width] = $style->buildUrl($uri);
          }
        }
      }
    }
    ksort($styles);
    return $styles;
  }

  protected function getFallBackStyle($uri) {
    if ( $this->getSetting('fallback_style') == 'original') {
      return file_create_url($uri);
    }

    $style = ImageStyle::load($this->getSetting('fallback_style'));
    if (!$style) {
      return file_create_url($uri);
    }

    return $style->buildUrl($uri);
  }

  /**
   * {@inheritdoc}
   */
  public function calculateDependencies() {
    $dependencies = parent::calculateDependencies();
    $styles[] = $this->getSetting('image_style_placeholder');
    $styles += $this->getSetting('image_styles');
    $styles[] = $this->getSetting('fallback_style');

    foreach ($styles as $style_id) {
      /** @var \Drupal\image\ImageStyleInterface $style */
      if ($style_id && $style = ImageStyle::load($style_id)) {
        // If this formatter uses a valid image style to display the image, add
        // the image style configuration entity as dependency of this formatter.
        $dependencies[$style->getConfigDependencyKey()][] = $style->getConfigDependencyName();
      }
    }
    return $dependencies;
  }

  /**
   * Generate the output appropriate for one field item.
   *
   * @param \Drupal\Core\Field\FieldItemInterface $item
   *   One field item.
   *
   * @return string
   *   The textual output generated.
   */
  protected function viewValue(FieldItemInterface $item) {
    $file = File::load($item->target_id);
    if (!$file) {
      return [];
    }
    $render = [
      '#theme' => 'image_style',
      '#style_name' => $this->getSetting('image_style_placeholder'),
      '#uri' => $file->getFileUri(),
      '#attributes' => [
        'data-src' => addslashes(json_encode($this->getStyles($file->getFileUri()))),
        'class' => [
          'image-sizes',
          'pre-load'
        ],
        'data-src-fallback' => $this->getFallBackStyle($file->getFileUri()),
      ],
      // optional parameters
    ];
    return $render;
  }

}
